#ifndef LIBUSEFUL_MATHEXPR_H
#define LIBUSEFUL_MATHEXPR_H

#include "includes.h"

#ifdef __cplusplus
extern "C" {
#endif

double EvaluateMathStr(char *String);


#ifdef __cplusplus
}
#endif



#endif
