#ifndef LIBUSEFUL_INCLUDES_H
#define LIBUSEFUL_INCLUDES_H

#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>
#include <syslog.h>
#include <stdlib.h>
#include <limits.h>

#include "defines.h"
#include "string.h"
#include "list.h"
#include "GeneralFunctions.h"
#include "file.h"
#include "socket.h"
#include "Vars.h"
#include "libsettings.h"

#endif
