#ifndef LIB_USEFUL_H
#define LIB_USEFUL_H

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdio.h>
#include "socket.h"
#include "unix_socket.h"
#include "string.h"
#include "expect.h"
#include "list.h"
#include "file.h"
#include "FileSystem.h"
#include "GeneralFunctions.h"
#include "DataProcessing.h"
#include "EncryptedFiles.h"
#include "ConnectManager.h"
#include "Hash.h"
#include "Compression.h"
#include "sound.h"
#include "pty.h"
#include "Log.h"
#include "http.h"
#include "oauth.h"
#include "Time.h"
#include "Vars.h"
#include "Markup.h"
#include "MathExpr.h"
#include "PatternMatch.h"

#endif
